@@ -18,7 +18,7 @@ use FileHandle ();
 use File::Basename ();
 use File::Path ();
 use File::Spec ();
-use vars qw($VERSION $urllist);
+use vars qw($VERSION $urllist $silent);
 $VERSION = "5.53";
 
 =head1 NAME
@@ -540,6 +540,16 @@ memory consumption of CPAN.pm considerably.
 
 Use CPAN::SQLite if available? (yes/no)?
 
+=item version_timeout
+
+This timeout prevents CPAN from hanging when trying to parse a
+pathologically coded $VERSION from a module.
+
+The default is 15 seconds.  If you set this value to 0, no timeout
+will occur, but this is not recommended.
+
+Timeout for parsing module versions?
+
 =item yaml_load_code
 
 Both YAML.pm and YAML::Syck are capable of deserialising code. As this
@@ -576,7 +586,6 @@ use vars qw( %prompts );
     my @prompts = (
 
 manual_config => qq[
-
 CPAN is the world-wide archive of perl resources. It consists of about
 300 sites that all replicate the same contents around the globe. Many
 countries have at least one CPAN site already. The resources found on
@@ -589,6 +598,7 @@ If you prefer to enter a dialog instead, you can answer 'no' to this
 question and I'll let you configure in small steps one thing after the
 other. (Note: you can revisit this dialog anytime later by typing 'o
 conf init' at the cpan prompt.)
+
 ],
 
 config_intro => qq{
@@ -643,12 +653,15 @@ session.
 },
 
 urls_intro => qq{
+Now we need to know where your favorite CPAN mirror sites are located. 
+You should select more than one (just in case the first isn't available).
+You can select from the global list of CPAN mirror sites or you can
+enter CPAN mirror URLs by hand.  A local CPAN mirror can be listed using
+a 'file:///' URL.  For example: 'file:///path/to/cpan/'
 
-Now we need to know where your favorite CPAN sites are located. Push
-a few sites onto the array (just in case the first on the array won\'t
-work). If you are mirroring CPAN to your local workstation, specify a
-file: URL.
+},
 
+urls_picker_intro => qq{
 First, pick a nearby continent and country by typing in the number(s)
 in front of the item(s) you want to select. You can pick several of
 each, separated by spaces. Then, you will be presented with a list of
@@ -770,38 +783,17 @@ sub init {
             $fastread = 0;
         } else {
             $fastread = 1;
+            $silent = 1;
             $CPAN::Config->{urllist} ||= [];
+            $CPAN::Config->{connect_to_internet_ok} ||= 1;
 
             local $^W = 0;
             # prototype should match that of &MakeMaker::prompt
             my $current_second = time;
             my $current_second_count = 0;
             my $i_am_mad = 0;
-            *_real_prompt = sub {
-                my($q,$a) = @_;
-                my($ret) = defined $a ? $a : "";
-                $CPAN::Frontend->myprint(sprintf qq{%s [%s]\n\n}, $q, $ret);
-                eval { require Time::HiRes };
-                unless ($@) {
-                    if (time == $current_second) {
-                        $current_second_count++;
-                        if ($current_second_count > 20) {
-                            # I don't like more than 20 prompts per second
-                            $i_am_mad++;
-                        }
-                    } else {
-                        $current_second = time;
-                        $current_second_count = 0;
-                        $i_am_mad-- if $i_am_mad>0;
-                    }
-                    if ($i_am_mad>0) {
-                        #require Carp;
-                        #Carp::cluck("SLEEEEEEEEPIIIIIIIIIIINGGGGGGGGGGG");
-                        Time::HiRes::sleep(0.1);
-                    }
-                }
-                $ret;
-            };
+            # silent prompting -- just quietly use default
+            *_real_prompt = sub { return $_[1] };
         }
     }
 
@@ -812,7 +804,7 @@ sub init {
                        keep_source_where
                        prefs_dir
                       } =~ /$matcher/) {
-        $CPAN::Frontend->myprint($prompts{config_intro});
+        $CPAN::Frontend->myprint($prompts{config_intro}) unless $silent;
 
         init_cpan_home($matcher);
 
@@ -876,9 +868,14 @@ sub init {
             $CPAN::META->has_inst("CPAN::Reporter") &&
             CPAN::Reporter->can('configure')
            ) {
-            $CPAN::Frontend->myprint("\nProceeding to configure CPAN::Reporter.\n");
-            CPAN::Reporter::configure();
-            $CPAN::Frontend->myprint("\nReturning to CPAN configuration.\n");
+            local *_real_prompt;
+            *_real_prompt = \&CPAN::Shell::colorable_makemaker_prompt;
+            my $_conf = prompt("Would you like me configure CPAN::Reporter now?", $silent ? "no" : "yes");
+            if ($_conf =~ /^y/i) {
+              $CPAN::Frontend->myprint("\nProceeding to configure CPAN::Reporter.\n");
+              CPAN::Reporter::configure();
+              $CPAN::Frontend->myprint("\nReturning to CPAN configuration.\n");
+            }
         }
     }
 
@@ -889,11 +886,14 @@ sub init {
     #
     if (!$matcher or "yaml_module" =~ /$matcher/) {
         my_dflt_prompt(yaml_module => "YAML", $matcher);
+        my $old_v = $CPAN::Config->{load_module_verbosity};
+        $CPAN::Config->{load_module_verbosity} = q[none];
         unless ($CPAN::META->has_inst($CPAN::Config->{yaml_module})) {
             $CPAN::Frontend->mywarn
                 ("Warning (maybe harmless): '$CPAN::Config->{yaml_module}' not installed.\n");
             $CPAN::Frontend->mysleep(3);
         }
+        $CPAN::Config->{load_module_verbosity} = $old_v;
     }
 
     #
@@ -927,7 +927,7 @@ sub init {
         if ($^O eq 'MacOS') {
             $CPAN::Config->{'shell'} = 'not_here';
         } else {
-            $path =~ s,\\,/,g if $^O eq 'os2'; # Cosmetic only
+            $path ||= 'sh', $path =~ s,\\,/,g if $^O eq 'os2'; # Cosmetic only
             my_dflt_prompt(shell => $path, $matcher);
         }
     }
@@ -938,7 +938,7 @@ sub init {
 
     my_prompt_loop(tar_verbosity => 'v', $matcher,
                    'none|v|vv');
-    my_prompt_loop(load_module_verbosity => 'v', $matcher,
+    my_prompt_loop(load_module_verbosity => 'none', $matcher,
                    'none|v');
     my_prompt_loop(perl5lib_verbosity => 'v', $matcher,
                    'none|v');
@@ -990,6 +990,7 @@ sub init {
     #
 
     my_dflt_prompt(inactivity_timeout => 0, $matcher);
+    my_dflt_prompt(version_timeout => 15, $matcher);
 
     #
     #== halt_on_failure
@@ -1003,7 +1004,7 @@ sub init {
     my @proxy_vars = qw/ftp_proxy http_proxy no_proxy/;
     my @proxy_user_vars = qw/proxy_user proxy_pass/;
     if (!$matcher or "@proxy_vars @proxy_user_vars" =~ /$matcher/) {
-        $CPAN::Frontend->myprint($prompts{proxy_intro});
+        $CPAN::Frontend->myprint($prompts{proxy_intro}) unless $silent;
 
         for (@proxy_vars) {
             $prompts{$_} = "Your $_?";
@@ -1015,21 +1016,21 @@ sub init {
 
             $default = $CPAN::Config->{proxy_user} || $CPAN::LWP::UserAgent::USER || "";
 
-            $CPAN::Frontend->myprint($prompts{proxy_user});
+            $CPAN::Frontend->myprint($prompts{proxy_user}) unless $silent;
 
             if ($CPAN::Config->{proxy_user} = prompt("Your proxy user id?",$default)) {
-                $CPAN::Frontend->myprint($prompts{proxy_pass});
+                $CPAN::Frontend->myprint($prompts{proxy_pass}) unless $silent;
 
                 if ($CPAN::META->has_inst("Term::ReadKey")) {
                     Term::ReadKey::ReadMode("noecho");
                 } else {
-                    $CPAN::Frontend->myprint($prompts{password_warn});
+                    $CPAN::Frontend->myprint($prompts{password_warn}) unless $silent;
                 }
                 $CPAN::Config->{proxy_pass} = prompt_no_strip("Your proxy password?");
                 if ($CPAN::META->has_inst("Term::ReadKey")) {
                     Term::ReadKey::ReadMode("restore");
                 }
-                $CPAN::Frontend->myprint("\n\n");
+                $CPAN::Frontend->myprint("\n\n") unless $silent;
             }
         }
     }
@@ -1058,25 +1059,25 @@ sub init {
         if ($CPAN::Config->{colorize_output}) {
             if ($CPAN::META->has_inst("Term::ANSIColor")) {
                 my $T="gYw";
-                print "                                      on_  on_y ".
-                    "        on_ma           on_\n";
-                print "                   on_black on_red  green ellow ".
-                    "on_blue genta on_cyan white\n";
+                $CPAN::Frontend->myprint( "                                      on_  on_y ".
+                    "        on_ma           on_\n") unless $silent;
+                $CPAN::Frontend->myprint( "                   on_black on_red  green ellow ".
+                    "on_blue genta on_cyan white\n") unless $silent;
 
                 for my $FG ("", "bold",
                             map {$_,"bold $_"} "black","red","green",
                             "yellow","blue",
                             "magenta",
                             "cyan","white") {
-                    printf "%12s ", $FG;
+                    $CPAN::Frontend->myprint(sprintf( "%12s ", $FG)) unless $silent;
                     for my $BG ("",map {"on_$_"} qw(black red green yellow
                                                     blue magenta cyan white)) {
-                        print $FG||$BG ?
-                            Term::ANSIColor::colored("  $T  ","$FG $BG") : "  $T  ";
+                            $CPAN::Frontend->myprint( $FG||$BG ?
+                            Term::ANSIColor::colored("  $T  ","$FG $BG") : "  $T  ") unless $silent;
                     }
-                    print "\n";
+                    $CPAN::Frontend->myprint( "\n" ) unless $silent;
                 }
-                print "\n";
+                $CPAN::Frontend->myprint( "\n" ) unless $silent;
             }
             for my $tuple (
                            ["colorize_print", "bold blue on_white"],
@@ -1106,7 +1107,7 @@ sub init {
     #
 
     if (!$matcher or 'histfile histsize' =~ /$matcher/) {
-        $CPAN::Frontend->myprint($prompts{histfile_intro});
+        $CPAN::Frontend->myprint($prompts{histfile_intro}) unless $silent;
         defined($default = $CPAN::Config->{histfile}) or
             $default = File::Spec->catfile($CPAN::Config->{cpan_home},"histfile");
         my_dflt_prompt(histfile => $default, $matcher);
@@ -1159,13 +1160,41 @@ sub init {
             my_dflt_prompt(ftpstats_period => 14, $matcher);
         }
     } elsif ($fastread) {
-        $CPAN::Frontend->myprint("Autoconfigured everything but 'urllist'.\n".
-                                 "Please call 'o conf init urllist' to configure ".
-                                 "your CPAN server(s) now!\n\n");
-    } else {
-        conf_sites();
+        $silent = 0;
+        local *_real_prompt;
+        *_real_prompt = \&CPAN::Shell::colorable_makemaker_prompt;
+        if ( @{ $CPAN::Config->{urllist} || [] } ) {
+            $CPAN::Frontend->myprint(
+              "\nYour 'urllist' is already configured. Type 'o conf init urllist' to change it.\n"
+            );
+        }
+        else {
+          $CPAN::Frontend->myprint(
+              "Autoconfigured everything but 'urllist'.\n"
+          );
+
+          $CPAN::Frontend->myprint($prompts{urls_intro});
+
+          my $_conf = prompt(
+            "Would you like to pick from the CPAN mirror list?", "yes"
+          );
+          $CPAN::Frontend->myprint("\n");
+
+          if ( $_conf =~ /^y/ ) {
+            conf_sites();
+          }
+          else {
+            bring_your_own();
+          }
+          $CPAN::Config->{urllist} = $urllist;
+        }
+        $CPAN::Frontend->myprint(
+            "\nAutoconfiguration complete.\n"
+        );
     }
 
+    $silent = 0; # reset
+
     $CPAN::Frontend->myprint("\n\n");
     if ($matcher && !$CPAN::Config->{auto_commit}) {
         $CPAN::Frontend->myprint("Please remember to call 'o conf commit' to ".
@@ -1188,7 +1217,7 @@ sub _init_external_progs {
                             patch applypatch
                             /;
     if (!$matcher or "@external_progs" =~ /$matcher/) {
-        $CPAN::Frontend->myprint($prompts{external_progs});
+        $CPAN::Frontend->myprint($prompts{external_progs}) unless $silent;
 
         my $old_warn = $^W;
         local $^W if $^O eq 'MacOS';
@@ -1229,7 +1258,7 @@ sub _init_external_progs {
             $path ||= find_exe($progcall,$PATH);
             unless ($path) { # not -e $path, because find_exe already checked that
                 local $"=";";
-                $CPAN::Frontend->mywarn("Warning: $progcall not found in PATH[@$PATH]\n");
+                $CPAN::Frontend->mywarn("Warning: $progcall not found in PATH[@$PATH]\n") unless $silent;
                 if ($progname eq "make") {
                     $CPAN::Frontend->mywarn("ALERT: 'make' is an essential tool for ".
                                             "building perl Modules. Please make sure you ".
@@ -1271,16 +1300,16 @@ I see you already have a  directory
     $cpan_home
 Shall we use it as the general CPAN build and cache directory?
 
-});
+}) unless $silent;
         } else {
             # no cpan-home, must prompt and get one
-            $CPAN::Frontend->myprint($prompts{cpan_home_where});
+            $CPAN::Frontend->myprint($prompts{cpan_home_where}) unless $silent;
         }
 
         my $default = $cpan_home;
         my $loop = 0;
         my($last_ans,$ans);
-        $CPAN::Frontend->myprint(" <cpan_home>\n");
+        $CPAN::Frontend->myprint(" <cpan_home>\n") unless $silent;
     PROMPT: while ($ans = prompt("CPAN build and cache directory?",$default)) {
             print "\n";
             if (File::Spec->file_name_is_absolute($ans)) {
@@ -1329,13 +1358,13 @@ sub my_dflt_prompt {
     my ($item, $dflt, $m) = @_;
     my $default = $CPAN::Config->{$item} || $dflt;
 
-    if (!$m || $item =~ /$m/) {
+    if (!$silent && (!$m || $item =~ /$m/)) {
         if (my $intro = $prompts{$item . "_intro"}) {
             $CPAN::Frontend->myprint($intro);
         }
         $CPAN::Frontend->myprint(" <$item>\n");
         $CPAN::Config->{$item} = prompt($prompts{$item}, $default);
-        print "\n";
+        $CPAN::Frontend->myprint("\n");
     } else {
         $CPAN::Config->{$item} = $default;
     }
@@ -1347,14 +1376,14 @@ sub my_yn_prompt {
     defined($default = $CPAN::Config->{$item}) or $default = $dflt;
 
     # $DB::single = 1;
-    if (!$m || $item =~ /$m/) {
+    if (!$silent && (!$m || $item =~ /$m/)) {
         if (my $intro = $prompts{$item . "_intro"}) {
             $CPAN::Frontend->myprint($intro);
         }
         $CPAN::Frontend->myprint(" <$item>\n");
         my $ans = prompt($prompts{$item}, $default ? 'yes' : 'no');
         $CPAN::Config->{$item} = ($ans =~ /^[y1]/i ? 1 : 0);
-        print "\n";
+        $CPAN::Frontend->myprint("\n");
     } else {
         $CPAN::Config->{$item} = $default;
     }
@@ -1365,13 +1394,13 @@ sub my_prompt_loop {
     my $default = $CPAN::Config->{$item} || $dflt;
     my $ans;
 
-    if (!$m || $item =~ /$m/) {
+    if (!$silent && (!$m || $item =~ /$m/)) {
         $CPAN::Frontend->myprint($prompts{$item . "_intro"});
         $CPAN::Frontend->myprint(" <$item>\n");
         do { $ans = prompt($prompts{$item}, $default);
         } until $ans =~ /$ok/;
         $CPAN::Config->{$item} = $ans;
-        print "\n";
+        $CPAN::Frontend->myprint("\n");
     } else {
         $CPAN::Config->{$item} = $default;
     }
@@ -1415,21 +1444,21 @@ Shall I use the local database in $mby?};
                 and $CPAN::Config->{connect_to_internet_ok})){
             if ($overwrite_local) {
                 $CPAN::Frontend->myprint(qq{Trying to overwrite $mby\n});
-                $better_mby = CPAN::FTP->localize($m,$mby,3);
+                $better_mby = CPAN::FTP->localize($m,$mby,3,1);
                 $overwrite_local = 0;
                 $use_mby=1 if $mby;
             } elsif ( ! -f $mby ) {
                 $CPAN::Frontend->myprint(qq{You have no $mby\n  I'm trying to fetch one\n});
-                $better_mby = CPAN::FTP->localize($m,$mby,3);
+                $better_mby = CPAN::FTP->localize($m,$mby,3,1);
                 $use_mby=1 if $mby;
             } elsif ( -M $mby > 60 ) {
                 $CPAN::Frontend->myprint(qq{Your $mby is older than 60 days,\n  I'm trying }.
                                          qq{to fetch a new one\n});
-                $better_mby = CPAN::FTP->localize($m,$mby,3);
+                $better_mby = CPAN::FTP->localize($m,$mby,3,1);
                 $use_mby=1 if $mby;
             } elsif (-s $mby == 0) {
                 $CPAN::Frontend->myprint(qq{You have an empty $mby,\n  I'm trying to fetch a better one\n});
-                $better_mby = CPAN::FTP->localize($m,$mby,3);
+                $better_mby = CPAN::FTP->localize($m,$mby,3,1);
                 $use_mby=1 if $mby;
             } else {
                 last LOOP;
@@ -1465,6 +1494,7 @@ a valid CPAN URL now.\n\n});
         $CPAN::Frontend->myprint("Current set of CPAN URLs:\n");
         map { $CPAN::Frontend->myprint("  $_\n") } @$urllist;
     }
+    $CPAN::Frontend->myprint("Now you have a chance to add any additional CPAN URLs\n");
     bring_your_own();
     $CPAN::Config->{urllist} = $urllist;
 }
@@ -1509,7 +1539,10 @@ sub picklist {
         }
         my $i = scalar @$items;
         unrangify(\@nums);
-        if (grep (/\D/ || $_ < 1 || $_ > $i, @nums)) {
+        if (0 == @nums) {
+            # cannot allow nothing because nothing means paging!
+            # return;
+        } elsif (grep (/\D/ || $_ < 1 || $_ > $i, @nums)) {
             $CPAN::Frontend->mywarn("invalid items entered, try again\n");
             if ("@nums" =~ /\D/) {
                 $CPAN::Frontend->mywarn("(we are expecting only numbers between 1 and $i)\n");
@@ -1522,7 +1555,10 @@ sub picklist {
         $CPAN::Frontend->myprint("\n");
 
         # a blank line continues...
-        next SELECTION unless @nums;
+        unless (@nums){
+            $CPAN::Frontend->mysleep(0.1); # prevent hot spinning process on the next bug
+            next SELECTION;
+        }
         last;
     }
     for (@nums) { $_-- }
@@ -1587,7 +1623,7 @@ sub read_mirrored_by {
     $CPAN::Config->{urllist} ||= [];
     my @previous_urls = @{$CPAN::Config->{urllist}};
 
-    $CPAN::Frontend->myprint($prompts{urls_intro});
+    $CPAN::Frontend->myprint($prompts{urls_picker_intro});
 
     my (@cont, $cont, %cont, @countries, @urls, %seen);
     my $no_previous_warn =
@@ -1597,13 +1633,17 @@ sub read_mirrored_by {
     if (@previous_urls) {
         push @$offer_cont, "(edit previous picks)";
         $default = @$offer_cont;
+    } else {
+        # cannot allow nothing because nothing means paging!
+        # push @$offer_cont, "(none of the above)";
     }
     @cont = picklist($offer_cont,
                      "Select your continent (or several nearby continents)",
                      $default,
                      ! @previous_urls,
                      $no_previous_warn);
-
+    # cannot allow nothing because nothing means paging!
+    # return unless @cont;
 
     foreach $cont (@cont) {
         my @c = sort keys %{$all{$cont}};
@@ -1646,7 +1686,11 @@ put them on one line, separated by blanks, hyphenated ranges allowed
 
     @urls = picklist (\@urls, $prompt, $default);
     foreach (@urls) { s/ \(.*\)//; }
-    push @$urllist, @urls;
+    if (@urls) {
+        $urllist = \@urls;
+    } else {
+        push @$urllist, @urls;
+    }
 }
 
 sub bring_your_own {
@@ -1692,7 +1736,7 @@ later if you\'re sure it\'s right.\n},
     @$urllist = CPAN::_uniq(@$urllist, @urls);
     $CPAN::Config->{urllist} = $urllist;
     # xxx delete or comment these out when you're happy that it works
-    $CPAN::Frontend->myprint("New set of picks:\n");
+    $CPAN::Frontend->myprint("New urllist\n");
     for ( @$urllist ) { $CPAN::Frontend->myprint("  $_\n") };
 }
 
