@@ -9,6 +9,11 @@ use vars qw(
 );
 $VERSION = "5.5";
 
+BEGIN {
+    # alarm() is not implemented in perl 5.6.x and earlier under Windows
+    *ALARM_IMPLEMENTED = sub () { $] >= 5.007 || $^O !~ /MSWin/ };
+}
+
 # Accessors
 #-> sub CPAN::Module::userid
 sub userid {
@@ -569,7 +574,7 @@ sub install {
     if ($ro && $ro->{stats} && $ro->{stats} eq "a") {
         $CPAN::Frontend->mywarn(qq{
 \n\n\n     ***WARNING***
-     The module $self->{ID} has no active maintainer.\n\n\n
+     The module $self->{ID} has no active maintainer (CPAN support level flag 'abandoned').\n\n\n
 });
         $CPAN::Frontend->mysleep(5);
     }
@@ -652,10 +657,20 @@ sub available_version {
 #-> sub CPAN::Module::parse_version ;
 sub parse_version {
     my($self,$parsefile) = @_;
-    my $have = eval { MM->parse_version($parsefile); };
+    if (ALARM_IMPLEMENTED) {
+        my $timeout = (exists($CPAN::Config{'version_timeout'}))
+                            ? $CPAN::Config{'version_timeout'}
+                            : 15;
+        alarm($timeout);
+    }
+    my $have = eval {
+        local $SIG{ALRM} = sub { die "alarm\n" };
+        MM->parse_version($parsefile);
+    };
     if ($@) {
         $CPAN::Frontend->mywarn("Error while parsing version number in file '$parsefile'\n");
     }
+    alarm(0) if ALARM_IMPLEMENTED;
     my $leastsanity = eval { defined $have && length $have; };
     $have = "undef" unless $leastsanity;
     $have =~ s/^ //; # since the %vd hack these two lines here are needed
