@@ -17,16 +17,32 @@ use vars qw(
            );
 @relo =     (
              "CPAN.pm",
+             "CPAN/Author.pm",
+             "CPAN/CacheMgr.pm",
+             "CPAN/Complete.pm",
              "CPAN/Debug.pm",
+             "CPAN/DeferredCode.pm",
+             "CPAN/Distribution.pm",
              "CPAN/Distroprefs.pm",
+             "CPAN/Distrostatus.pm",
+             "CPAN/Exception/RecursiveDependency.pm",
+             "CPAN/Exception/yaml_not_installed.pm",
              "CPAN/FirstTime.pm",
+             "CPAN/FTP.pm",
+             "CPAN/FTP/netrc.pm",
              "CPAN/HandleConfig.pm",
+             "CPAN/Index.pm",
+             "CPAN/InfoObj.pm",
              "CPAN/Kwalify.pm",
+             "CPAN/LWP/UserAgent.pm",
+             "CPAN/Module.pm",
+             "CPAN/Prompt.pm",
              "CPAN/Queue.pm",
              "CPAN/Reporter/Config.pm",
              "CPAN/Reporter/History.pm",
              "CPAN/Reporter/PrereqCheck.pm",
              "CPAN/Reporter.pm",
+             "CPAN/Shell.pm",
              "CPAN/SQLite.pm",
              "CPAN/Tarzip.pm",
              "CPAN/Version.pm",
@@ -255,6 +271,7 @@ sub globls {
                 $author->$pragma();
             }
         }
+        CPAN->debug("author[$author]pathglob[$pathglob]silent[$silent]") if $CPAN::DEBUG;
         push @results, $author->ls($pathglob,$silent); # silent if
                                                        # more than one
                                                        # author
@@ -1185,7 +1202,7 @@ sub autobundle {
     my($fh) = FileHandle->new(">$to") or Carp::croak "Can't open >$to: $!";
     $fh->print(
                "package Bundle::$me;\n\n",
-               "\$VERSION = '0.01';\n\n",
+               "\$","VERSION = '0.01';\n\n", # hide from perl-reversion
                "1;\n\n",
                "__END__\n\n",
                "=head1 NAME\n\n",
@@ -1609,10 +1626,22 @@ sub rematein {
             if (substr($s,-1,1) eq ".") {
                 $obj = CPAN::Shell->expandany($s);
             } else {
-                $CPAN::Frontend->mywarn("Sorry, $meth with a regular expression is ".
-                                        "not supported.\nRejecting argument '$s'\n");
-                $CPAN::Frontend->mysleep(2);
-                next;
+                my @obj;
+            CLASS: for my $class (qw(Distribution Bundle Module)) {
+                    if (@obj = $self->expand($class,$s)) {
+                        last CLASS;
+                    }
+                }
+                if (@obj) {
+                    if (1==@obj) {
+                        $obj = $obj[0];
+                    } else {
+                        $CPAN::Frontend->mywarn("Sorry, $meth with a regular expression is ".
+                                                "only supported when unambiguous.\nRejecting argument '$s'\n");
+                        $CPAN::Frontend->mysleep(2);
+                        next STHING;
+                    }
+                }
             }
         } elsif ($meth eq "ls") {
             $self->globls($s,\@pragma);
