@@ -4,7 +4,7 @@ use strict;
 use vars qw($VERSION @ISA $BUGHUNTING);
 use CPAN::Debug;
 use File::Basename qw(basename);
-$VERSION = "5.5";
+$VERSION = "5.501";
 # module is internal to CPAN.pm
 
 @ISA = qw(CPAN::Debug); ## no critic
@@ -223,17 +223,32 @@ sub untar {
     $exttar = "" if $exttar =~ /^\s+$/; # user refuses to use it
     my $extgzip = $self->{UNGZIPPRG} || "";
     $extgzip = "" if $extgzip =~ /^\s+$/; # user refuses to use it
+
     if (0) { # makes changing order easier
     } elsif ($BUGHUNTING) {
         $prefer=2;
-    } elsif ($exttar && $extgzip) {
-        # should be default until Archive::Tar handles bzip2
+    } elsif ($exttar && $extgzip && $file =~ /\.bz2$/i) {
+        # until Archive::Tar handles bzip2
         $prefer = 1;
     } elsif (
              $CPAN::META->has_usable("Archive::Tar")
              &&
              $CPAN::META->has_inst("Compress::Zlib") ) {
         $prefer = 2;
+    } elsif ($exttar && $extgzip) {
+        # no modules and not bz2
+        $prefer = 1;
+        # but solaris binary tar is a problem
+        if ($^O eq 'solaris' && qx($exttar --version 2>/dev/null) !~ /gnu/i) {
+            $CPAN::Frontend->mywarn(<< 'END_WARN');
+
+WARNING: Many CPAN distributions were archived with GNU tar and some of
+them may be incompatible with Solaris tar.  We respectfully suggest you
+configure CPAN to use a GNU tar instead ("o conf init tar") or install
+a recent Archive::Tar instead;
+
+END_WARN
+        }
     } else {
         my $foundtar = $exttar ? "'$exttar'" : "nothing";
         my $foundzip = $extgzip ? "'$extgzip'" : $foundtar ? "nothing" : "also nothing";
@@ -308,9 +323,12 @@ Can't continue cutting file '$file'.
         unless ($CPAN::META->has_usable("Archive::Tar")) {
             $CPAN::Frontend->mydie("Archive::Tar not installed, please install it to continue");
         }
-        # Make sure AT does not use permissions in the archive
+        # Make sure AT does not use uid/gid/permissions in the archive
         # This leaves it to the user's umask instead
-        local $Archive::Tar::CHMOD = 0;
+        local $Archive::Tar::CHMOD = 1;
+        local $Archive::Tar::SAME_PERMISSIONS = 0;
+        # Make sure AT leaves current user as owner
+        local $Archive::Tar::CHOWN = 0;
         my $tar = Archive::Tar->new($file,1);
         my $af; # archive file
         my @af;
