@@ -28,6 +28,7 @@ $CPAN::Config = {
                  'inhibit_startup_message' => q[0],
                  'keep_source_where' => qq[$cwd/t/dot-cpan/sources],
                  #'lynx' => q[],
+                 'load_module_verbosity' => q[v],
                  #'make' => q[/usr/bin/make],
                  'make_arg' => q[],
                  'make_install_arg' => q[UNINST=1],
@@ -51,6 +52,7 @@ $CPAN::Config = {
                  'urllist' => [qq[file://$cwd/t/CPAN]],
                  'wait_list' => [q[wait://ls6.informatik.uni-dortmund.de:1404]],
                  #'wget' => q[/usr/bin/wget],
+                 'yaml_load_code' => q[0],
                 };
 
 __END__
